/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/
#include <FBManagerStates.h>
#include <FBManagerStateInterface.h>
#include <FBManagerInterface.h>
#include <qdebug.h>


// `IdleBMState` state means the buffer manager is idle and no buffer is being written. i.e. its not waiting for fb update.
IdleBMState::IdleBMState(FBManager *fbManager){
    mFbManager = fbManager;
}

IdleBMState::~IdleBMState(){
    mFbManager = NULL;
}

void IdleBMState::onBufferNormalStateChange(){

    FrameBuffer * buff = NULL;

    //check for IDLE buffer availability
    buff = mFbManager->getFrameBufferByState(FB_STATE_IDLE);
    if(buff != NULL)
    {
        qDebug()<<"DOUBLE_BUFFER_LOG : Changing state from IDLE -> AWAITING of BUFFER MANAGER";
        mFbManager->setCurrentState(mFbManager->getAwaitingFBUpdateBMState());
        mFbManager->startWritingToIdleBuffer();
    }
    else
    {
        qDebug()<<"DOUBLE_BUFFER_LOG : IDLE BUFFER NOT AVAILABLE";
    }
}

void IdleBMState::onBufferAbnormalStateChange(){
    qDebug() << "DOUBLE_BUFFER_LOG : This should not occur !!"<<__PRETTY_FUNCTION__;
    // do error handling
}


// `AwaitingFBUpdateBMState` state means the Buffer manager is waiting for the fb update.
AwaitingFBUpdateBMState::AwaitingFBUpdateBMState(FBManager *fbManager){
    mFbManager = fbManager;
}

AwaitingFBUpdateBMState::~AwaitingFBUpdateBMState(){
    mFbManager = NULL;
}

void AwaitingFBUpdateBMState::onBufferNormalStateChange(){
    // nothing to be done, already writing.
    qDebug() << "DOUBLE_BUFFER_LOG : BUFFER MANAGER is AWAITING";
}

void AwaitingFBUpdateBMState::onBufferAbnormalStateChange(){
    qDebug() << "DOUBLE_BUFFER_LOG : This should not occur !!"<<__PRETTY_FUNCTION__;
    // do error handling..
}

