/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/
#include <FBStates.h>
#include <FrameBuffer.h>
#include <QDebug>

// the idle state of the frame buffer
IdleFBState::IdleFBState(FrameBuffer *frameBuffer, int stateId){
    mFrameBuffer = frameBuffer;
    mStateId = stateId;
}

IdleFBState::~IdleFBState(){
    mFrameBuffer = NULL;
}

// prepare the buffer for the writing
void IdleFBState::prepareForWriting(){
    // set the current state of the fb state machine to writing state
    qDebug()<<"DOUBLE_BUFFER_LOG : Changing state from IDLE -> WRITING of buffer : "<<mFrameBuffer->getId();
    mFrameBuffer->setState(mFrameBuffer->getWritingState());
}

void IdleFBState::prepareForReading(){
    qDebug()<<"something is wrong, can't go directly to reading state from idle state.. taking necessary actions..";
}

void IdleFBState::prepareForIdle(){
    qDebug()<<"already idle..";
}

int IdleFBState::getStateId(){
    return mStateId;
}

// the writing state of the frame buffer
WritingFBState::WritingFBState(FrameBuffer *frameBuffer, int stateId){
    mFrameBuffer = frameBuffer;
    mStateId = stateId;
}

WritingFBState::~WritingFBState(){
    mFrameBuffer = NULL;
}

void WritingFBState::prepareForWriting(){
    qDebug()<<"already writing..";
}

void WritingFBState::prepareForReading(){
    qDebug()<<"DOUBLE_BUFFER_LOG : Changing state from WRITING -> READING of buffer : "<<mFrameBuffer->getId();
    mFrameBuffer->setState(mFrameBuffer->getReadingState());
}

void WritingFBState::prepareForIdle(){
    qDebug()<<"something is wrong, can't go directly to idle state from writing state.. taking necessary actions..";
}

int WritingFBState::getStateId(){
    return mStateId;
}

// the reading state of the frame buffer
ReadingFBState::ReadingFBState(FrameBuffer *frameBuffer, int stateId){
    mFrameBuffer = frameBuffer;
    mStateId = stateId;
}

void ReadingFBState::prepareForWriting(){
    qDebug()<<"something is wrong, can't go directly to writing state from reading state.. taking necessary actions..";
}

ReadingFBState::~ReadingFBState(){
    mFrameBuffer = NULL;
}

void ReadingFBState::prepareForReading(){
    qDebug()<<"already reading..";
}

void ReadingFBState::prepareForIdle(){
    qDebug()<<"DOUBLE_BUFFER_LOG : Changing state from READING -> IDLE of buffer : "<<mFrameBuffer->getId();
    mFrameBuffer->setState(mFrameBuffer->getIdleState());
}

int ReadingFBState::getStateId(){
    return mStateId;
}
