/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/
#include <FrameBuffer.h>
#include <FBManagerInterface.h>
#include <qdebug.h>


FrameBuffer::FrameBuffer(FBManager * ntfier, int bufferId){
        mNotifier = ntfier;
        mBufferSize = 0;
        mIdleState    = new IdleFBState(this, FB_STATE_IDLE);
        mWritingState = new WritingFBState(this, FB_STATE_WRITING);
        mReadingState = new ReadingFBState(this, FB_STATE_READING);
        mBufferId = bufferId;
        mBuffer = NULL;
        mCurrBuffState = mIdleState;
    }

FrameBuffer::~FrameBuffer(){
    mCurrBuffState = NULL;
    mNotifier = NULL;
    mBufferSize = 0;

    // the states of this machine
    delete  mIdleState;
    delete  mWritingState;
    delete  mReadingState;

    deAllocateBuffer();
}

int FrameBuffer::getId(){
    return mBufferId;
}

FBState * FrameBuffer::getIdleState(){
    return mIdleState;
}

FBState * FrameBuffer::getWritingState(){
    return mWritingState;
}

FBState * FrameBuffer::getReadingState(){
    return mReadingState;
}

void FrameBuffer::setStateChangeNotifier(FBManager * notifier)
{
    mNotifier = notifier;
}


// this method will allocate the frame buffer.
unsigned char * FrameBuffer::allocateBuffer(int size){
	mBuffer = new unsigned char[size];
    memset(mBuffer, 0, size);
    mBufferSize = size;
    return mBuffer;
}

unsigned char * FrameBuffer::deAllocateBuffer(){
    delete[] mBuffer;
    mBuffer = NULL;
}

unsigned char * FrameBuffer::getDataPtr(){
    return mBuffer;
}

FB_STATE FrameBuffer::getBufferStateId()
{
    if(mCurrBuffState!=NULL){
        return ((FB_STATE)mCurrBuffState->getStateId());
    }
    else{
        return FB_STATE_IDLE;
    }
}


/******************************* STATE MACHINE METHODS *******************************/
// fb state methods

void FrameBuffer::prepareForWriting(){
    mCurrBuffState->prepareForWriting();
}

void FrameBuffer::prepareForReading(){
    mCurrBuffState->prepareForReading();
}

void FrameBuffer::prepareForIdle(){
    mCurrBuffState->prepareForIdle();
}

// changing the state of the state machine
void FrameBuffer::setState(FBState *state){
    mCurrBuffState = state;
    mNotifier->onBufferNormalStateChange();
}

void FrameBuffer::reset(){
    mCurrBuffState = mIdleState;
    memset(mBuffer, 0, mBufferSize);
}
