/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/
/*
 * ChangeLog -  
 *				March 19, 2013
 * 				1. Added 'shutdown' for socket.
 */
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
//#include <ifaddrs.h>
#include <QtCore>
#include <TmHeaders/TCPSocket.h>
#include <TmHeaders/QTmGlobal.h>
#include <TmHeaders/libinterface.h>
#include <TmHeaders/appinterface.h>

#define PIPE_RD_INDX 0
#define PIPE_WR_INDX 1

TCPSocket::TCPSocket():m_State(PRE_INIT)
{
	m_tcp_socket = socket(AF_INET,SOCK_STREAM,0);        
	if(0 < m_tcp_socket)
	{
            m_pipe_fds[PIPE_RD_INDX] = NULL;
            m_pipe_fds[PIPE_WR_INDX] = NULL;
            int initStatus = pipe(m_pipe_fds);
            if(initStatus)
            {
                    qDebug() << "Error ocurred while opening pipe";
                    shutdown(m_tcp_socket,SHUT_RDWR);
                    close(m_tcp_socket);
            }
            else
            {
                m_State = CREATED;
                m_readState = STOPPED;
            }
	}

}

TCPSocket::~TCPSocket()
{
	switch(m_State)
	{
		case PRE_INIT:
			break;
		case CREATED:
			close(m_tcp_socket);
			break;
		case CONNECTED:
			closeSocket();
			break;
		case CLOSED:
			break;
	}

        closePipe();
}

int TCPSocket::connectToHost(const char* hostName, int port)
{
    int status = (int)AppInterface::FAILURE;
	if(CREATED == m_State)
	{
		struct sockaddr_in serv_addr;
		struct hostent *server = NULL;		
		server = gethostbyname(hostName);
		if(NULL != server)
		{
			bzero((char *) &serv_addr, sizeof(serv_addr));
			serv_addr.sin_family = AF_INET;
                        serv_addr.sin_port = htons(port);
                        //bcopy((char *)server->h_addr, (char *)&serv_addr.sin_addr.s_addr, server->h_length);
                        for(int byteCount = 0; byteCount < (server->h_length); byteCount++)
                        {
                            *(((char *)&serv_addr.sin_addr.s_addr) + byteCount) = *(((char *)server->h_addr) + byteCount);
                        }

                        //set SO_REUSEADDR on a socket to true (1):
                        int optval = 1;
                        setsockopt(m_tcp_socket, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof optval);

                        qDebug() << "TCPSocket connecting .." << hostName << port;
			status = connect(m_tcp_socket,(struct sockaddr *)&serv_addr,sizeof(struct sockaddr_in));
			if(!status)
			{
				struct timeval timeout;
				timeout.tv_sec = 60;
				timeout.tv_usec = 0;

                                qDebug() << "TCPSocket setsockopt ..";
                                fcntl(m_tcp_socket, F_SETFL, O_NONBLOCK);           //<<for ML cer
                                status = setsockopt (m_tcp_socket, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout, sizeof(timeout));
				if(!status)
				{
						status = setsockopt (m_tcp_socket, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout, sizeof(timeout));
				}
				if(!status)
				{
						m_State = CONNECTED;
				}
			}
		}
	}
	else
	{
            qDebug() << "TCPSocket ERR_INVALID_STATE";
            status = ERR_INVALID_STATE;
            shutdown(m_tcp_socket,SHUT_RDWR);
            close(m_tcp_socket);
            closePipe();
	}
	return (status);
}

int TCPSocket::connectToIp(const char* hostIp, int port)
{
    int status = (int)AppInterface::FAILURE;
        if(CREATED == m_State)
        {

            struct sockaddr_in serv_addr;
            bzero((char *) &serv_addr, sizeof(serv_addr));
            serv_addr.sin_family = AF_INET;
            serv_addr.sin_port = htons(port);
            serv_addr.sin_addr.s_addr=inet_addr(hostIp);
            bzero(&(serv_addr.sin_zero),8);

            //bind(m_tcp_socket,(struct sockaddr *)&serv_addr,sizeof(serv_addr));

            status = connect(m_tcp_socket,(struct sockaddr *)&serv_addr,sizeof(struct sockaddr_in));
            if(!status)
            {
                struct timeval timeout;
                timeout.tv_sec = 120;
                timeout.tv_usec = 0;

                status = setsockopt (m_tcp_socket, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout, sizeof(timeout));
                if(!status)
                {
                    status = setsockopt (m_tcp_socket, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout, sizeof(timeout));
                }
                if(!status)
                {
                    m_State = CONNECTED;
                }
            }
        }
        else
        {
                status = ERR_INVALID_STATE;
                shutdown(m_tcp_socket,SHUT_RDWR);
                close(m_tcp_socket);
                closePipe();

        }
        return (status);
}

int TCPSocket::socketWrite(void *wdata, int len)
{
    int status = (int)AppInterface::FAILURE;
    unsigned char* data = (unsigned char*)wdata;
    if(CONNECTED == m_State)
    {
        int totalBytes =0;
        int writtenBytes = 0;
        for(;totalBytes < len;)
        {
            writtenBytes = write(m_tcp_socket,(data + writtenBytes),len - totalBytes);
            if(writtenBytes == -1)
            {
                    break;
            }
            else
            {
                    totalBytes += writtenBytes;
            }
            status = totalBytes;
        }
    }
    else
    {
        status = ERR_INVALID_STATE;
        shutdown(m_tcp_socket,SHUT_RDWR);
        close(m_tcp_socket);
        closePipe();
    }
#ifdef QT_UI_CLEANUP_DEBUG_SOCKET
    qDebug()<< "TCPSocket::socketWrite() data written :" << status;
#endif
    return (status);
}

int TCPSocket::socketRead(void *rdata, int len)
{
    unsigned char* data = (unsigned char*)rdata;
    int status =(int)AppInterface::FAILURE;
	if(CONNECTED == m_State)
	{
		int totalBytes =0;
		int readBytes = 0;
                int numReady = 0;
                fd_set readSet;

                struct timeval selTimeout;
                selTimeout.tv_sec = 120;
                selTimeout.tv_usec = 0;

                FD_ZERO(&readSet);
                FD_SET(m_tcp_socket, &readSet);
                FD_SET(m_pipe_fds[PIPE_RD_INDX], &readSet);

                int nfds = (m_tcp_socket > m_pipe_fds[PIPE_RD_INDX]) ? m_tcp_socket : m_pipe_fds[PIPE_RD_INDX];

                m_readState = WAITING;

                //qDebug() << "socketRead() : Waiting for recv data...";

                //numReady = select(nfds+1, &readSet, NULL, NULL, &selTimeout);
                numReady = select(nfds+1, &readSet, NULL, NULL, NULL);
                if(numReady > 0)
                {
                    if(FD_ISSET(m_tcp_socket, &readSet))
                    {
                        m_readState = READING;
                        for(;totalBytes < len;)
                        {
                            readBytes = read(m_tcp_socket,(data + totalBytes),len - totalBytes);
                            if(readBytes == -1)
                            {
                                if(errno != EAGAIN)
                                {
                                    qDebug() << "TCPSocket::socketRead() read() returned" << readBytes;
                                    qDebug() << "TCPSocket::socketRead() read() error" << errno;
                                    m_readState = STOPPED;
                                    return AppInterface::FAILURE;
                                    //break;
                                }
                            }
                            else
                            {
                                    totalBytes += readBytes;
                            }
                        }
                    }
                    else if(FD_ISSET(m_pipe_fds[PIPE_RD_INDX], &readSet))
                    {
                        qDebug() << "force exit from 'select' on socket  !!";
                        m_readState = STOPPED;
                        return AppInterface::FAILURE;
                    }
                    else
                    {
                        qDebug() << "Unknown event exited select() !!";
                        m_readState = STOPPED;
                        return AppInterface::FAILURE;
                    }
                }
                else if(numReady == 0)
                {
                    qDebug() << "select timeout !!";
                    m_readState = STOPPED;
                    return AppInterface::FAILURE;
                }
                else if(numReady == -1)
                {
                    qDebug() << "select failure !!";
                    m_readState = STOPPED;
                    return AppInterface::FAILURE;
                }
		status = totalBytes;
	}
	else
	{
            qDebug() << "socketRead : ERR_INVALID_STATE";
            status = ERR_INVALID_STATE;
            shutdown(m_tcp_socket,SHUT_RDWR);
            close(m_tcp_socket);
            closePipe();
	}

        m_readState = STOPPED;
	return (status);
}

int TCPSocket::socketReadOnce(QByteArray& rdata, int *rdLen)
{
    int status =(int)AppInterface::FAILURE;
    char rcv_buff[20*1024] = {0};

        if(CONNECTED == m_State)
        {
            struct timeval selTimeout;
            selTimeout.tv_sec = 60;
            selTimeout.tv_usec = 0;

            int numReady = 0;
            int readBytes = 0;

            FD_ZERO(&readSet);
            FD_SET(m_tcp_socket, &readSet);
            FD_SET(m_pipe_fds[PIPE_RD_INDX], &readSet);

            int nfds = (m_tcp_socket > m_pipe_fds[PIPE_RD_INDX]) ? m_tcp_socket : m_pipe_fds[PIPE_RD_INDX];

            //qDebug() << "socketReadOnce() : Waiting for recv data...";
            m_readState = WAITING;
            numReady = select(nfds+1, &readSet, NULL, NULL, &selTimeout);
            qDebug() << "Got recv data...";
            if(numReady > 0)
            {
                if(FD_ISSET(m_tcp_socket, &readSet))
                {
                    m_readState = READING;
                    readBytes = read(m_tcp_socket, rcv_buff, sizeof(rcv_buff));
                    qDebug() << "recv data bytes: " << readBytes;
                    if(readBytes == -1)
                    {
                        qDebug() << "read error !!";
                        return (int)AppInterface::FAILURE;
                    }
                    else if(readBytes == 0)
                    {
                        qDebug() << "no data to read!!";
                        status = (int)AppInterface::FAILURE;
                    }
                    *rdLen = readBytes;
                    rdata.insert(0, rcv_buff, readBytes);
                    status = (int)AppInterface::SUCCESS;
                }
                else if(FD_ISSET(m_pipe_fds[PIPE_RD_INDX], &readSet))
                {
                    qDebug() << "force exit from 'select' on socket  !!";
                    status = (int)AppInterface::FAILURE;
                }
                else
                {
                    qDebug() << "Unknown event exited select() !!";
                    status = (int)AppInterface::FAILURE;
                }
            }
            else if(numReady == 0)
            {
                qDebug() << "select timeout !!";
                status = (int)AppInterface::FAILURE;
            }
            else if(numReady == -1)
            {
                qDebug() << "select failure !!";
                status = (int)AppInterface::FAILURE;
            }
        }
        else
        {
                status = ERR_INVALID_STATE;
                shutdown(m_tcp_socket,SHUT_RDWR);
                close(m_tcp_socket);
                closePipe();
        }

        m_readState = STOPPED;
        return (status);
}

int TCPSocket::closeSocket()
{
        int status = (int)AppInterface::FAILURE;
	if(CONNECTED == m_State)
	{
		status = shutdown(m_tcp_socket,SHUT_RDWR);
		if(!status)
		{
                    qDebug() << "TCPSocket :: close(m_tcp_socket)";
                    status = close(m_tcp_socket);
                    m_State = CLOSED;
		}
	}
	else
	{
            qDebug() << "TCPSocket :: closeSocket()";
            status = ERR_INVALID_STATE;
	}
	return (status);
}



void TCPSocket::closePipe()
{
    qDebug() << "TCPSocket :: closePipe()";
    if(m_pipe_fds[PIPE_RD_INDX] != NULL)
    {
        close(m_pipe_fds[PIPE_RD_INDX]);
        m_pipe_fds[PIPE_RD_INDX] = NULL;
    }
    if(m_pipe_fds[PIPE_WR_INDX] != NULL)
    {
        close(m_pipe_fds[PIPE_WR_INDX]);
        m_pipe_fds[PIPE_WR_INDX] = NULL;
    }
}

int TCPSocket::forceStopSocketRead()
{
    int retVal = (int)AppInterface::FAILURE;
    int dummyData = 0;

    qDebug() << "forceStopSocketRead() m_readState : " << m_readState;

    if((m_pipe_fds[PIPE_WR_INDX] != NULL) && (m_readState == WAITING))
    {
        retVal = write(m_pipe_fds[PIPE_WR_INDX],&dummyData,1);
        if(retVal < 0)
        {
            qDebug() << "forceStopSocketRead() error !!";
            retVal = (int)AppInterface::FAILURE;
        }
        else
        {
            qDebug() << "forceStopSocketRead() initiated !!";
            retVal = (int)AppInterface::SUCCESS;
        }
    }
    else if(m_readState == READING)                     //<<for ML cer
    {
        qDebug() << "forceStopSocketRead() closing socket !!";
        shutdown(m_tcp_socket,SHUT_RDWR);
        close(m_tcp_socket);
        retVal = (int)AppInterface::SUCCESS;
    }
    else
    {
        qDebug() << "forceStopSocketRead() not needed";
        retVal = (int)AppInterface::SUCCESS;
    }

    return retVal;
}

int TCPSocket::readAll()            //<<for ML cer
{
    int readBytes = 0;
    int dummy = 0;
    int totalCount = 0;

    qDebug() << "readall()  m_readState : " << m_readState;

    do
    {
        if(m_readState == STOPPED)
        readBytes = read(m_tcp_socket, &dummy, 1);
        totalCount ++;
    }
    while(readBytes > 0);

    qDebug() << "readall()  totalCount : " << totalCount;
}
