/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/
#ifndef FBSTATES_H
#define FBSTATES_H

#include <FBStateInterface.h>

class FrameBuffer;

// the idle state of the frame buffer
class IdleFBState : public FBState{
private:
    FrameBuffer *mFrameBuffer;
    int mStateId;

public:
    IdleFBState(FrameBuffer *frameBuffer, int stateId);

    ~IdleFBState();

    // prepare the buffer for the writing
    void prepareForWriting();

    void prepareForReading();

    void prepareForIdle();

    int getStateId();
};


// the writing state of the frame buffer
class WritingFBState : public FBState{
private:
    FrameBuffer *mFrameBuffer;
    int mStateId;

public:
    WritingFBState(FrameBuffer *frameBuffer, int stateId);

    ~WritingFBState();

    void prepareForWriting();

    void prepareForReading();

    void prepareForIdle();

    int getStateId();

};

// the reading state of the frame buffer
class ReadingFBState : public FBState{
private:
    FrameBuffer *mFrameBuffer;
    int mStateId;

public:
    ReadingFBState(FrameBuffer *frameBuffer, int stateId);

    ~ReadingFBState();

    void prepareForWriting();

    void prepareForReading();

    void prepareForIdle();

    int getStateId();
};

#endif // FBSTATES_H
