/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/
#ifndef FRAMEBUFFER_H
#define FRAMEBUFFER_H
#include <FBStateInterface.h>
#include <FBStates.h>

#define MAX_FRAME_SIZE (1080*720*2)

typedef enum{
    FB_STATE_IDLE,
    FB_STATE_WRITING,
    FB_STATE_READING,

    FB_STATE_MAX,
}FB_STATE;

// forward declaration
class FBManager;


class FrameBuffer{
private:
//    unsigned char buffer[MAX_FRAME_SIZE];
    unsigned char* mBuffer;
    FBState * mCurrBuffState;
    FBManager* mNotifier;
    int mBufferId;
    // the states of this machine
    IdleFBState *mIdleState;
    WritingFBState *mWritingState;
    ReadingFBState *mReadingState;
    int mBufferSize;

public:
    FrameBuffer(FBManager* ntfier, int bufferId);

    ~FrameBuffer();

    FBState *getIdleState();

    FBState *getWritingState();

    FBState *getReadingState();

    // this method will allocate the frame buffer.
    unsigned char * allocateBuffer(int size);
    unsigned char * deAllocateBuffer();

    // this method wil return the frame buffer pointer.
    unsigned char * getDataPtr();

    void setStateChangeNotifier(FBManager * notifier);

    FB_STATE getBufferStateId();

    /******************************* STATE MACHINE METHODS *******************************/
    // fb state methods
    void prepareForWriting();

    void prepareForReading();

    void prepareForIdle();

    // changing the state of the state machine
    void setState(FBState *state);
    /******************************* STATE MACHINE METHODS *******************************/

    void sendFBUpdate();

    int getId();

    void reset();
};

#endif //FRAMEBUFFER_H
