/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA 02111-1307, USA.
**
****************************************************************************/

/*
 * ChangeLog -  
 *				March 19, 2013
 * 				1. Added observer methods for the supporting double buffer implementation.
 */

#ifndef QTMSURFACE_H
#define QTMSURFACE_H

#ifndef QT_UI_CLEANUP
#include <QTransform>
#endif

#include "QTmGlobal.h"

/*@!{*/
#include "SurfaceObserver.h"
/*@!}*/

class QTmSurface
{
public:
    virtual uchar* allocateSurface(const QSize& s, QTmGlobal::TmColorFormat colorFormat, double xScalingFactor, double yScalingFactor) = 0;
    virtual void deallocateSurface() = 0;
    virtual void updateSurface(const QRect& updateRect) = 0;

    /*@!{*/
    virtual void registerObserver(SurfaceObserver *) = 0;
    virtual void notify() = 0;
    virtual unsigned char * getSurface()=0;
    virtual void start() = 0;
    virtual void resetSurface() = 0;
    /*@!}*/

#ifndef QT_UI_CLEANUP
    static int translateKey(int key, Qt::KeyboardModifiers modifier);
    static QTransform getTransformation(QSize viewSize, QSize surfaceSize);
#endif

};

#endif // QTMSURFACE_H
