/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/

#ifndef LIBINTERFACEIMPL_H
#define LIBINTERFACEIMPL_H



#include <QtCore>
#include <TmHeaders/QTmGlobal.h>
#include <TmHeaders/QTmSurface.h>
#include <TmHeaders/libinterface.h>

class QTmClient;
class AppInterface;
class QTmRemoteServer;
class QTmApplication;
class QTmClientProfile;
class QTmAppInfo;
class QTmDisplayInfo;
class LibInterfaceEmitter;

/**
This class implements the interface LibInterface.
All the methods are actually the slots those get executed after
signal emission from LibInterfaceEmitter class.
This is done to ensure the execution of the TM JNI API calls through
inside Qt event loop and not from calling thread.
It also reports the success/failure of the API through AppInterface.
*/

class LibInterfaceImpl:public QObject,public LibInterface
{
    Q_OBJECT
private:

    AppInterface                            *m_appInterface; // This will be the response handler thread
    QTmClient                               *m_TerminalMode;
    QList<QTmRemoteServer*>                 *m_remoteServerList;
    QMultiHash<int, int>                    *m_remoteServerIdToVncIdMap;
    QMultiHash<QString, QTmApplication*>    *m_appIdToApplicationMap;
    QHash<int, QTmSurface* >                *m_surfaceMap;

    QTmApplication* getApplicationById(char* id);
    QTmRemoteServer* getRemoteServer(int id);
public:
    LibInterfaceImpl(LibInterfaceEmitter* emitter);
    ~LibInterfaceImpl();
    void setAppInterface(AppInterface* appInterface);

public slots:

    /** ------------   methods inherited from LibInterface   --------------------**/

    int upnpStartControlPoint();
    int upnpStopControlPoint();
    int upnpConnectControlPoint(char* interfaceName, char* address);

    int upnpDisconnectControlPoint(char* hostAddress);
    int upnpGetApplicationStatus(char*  appId);
    int upnpLaunchApplication(char* appId);
    int upnpTerminateApplication(char* appId);

    int verifyTmClientProfile(QTmClientProfile *clientProfile, int profileId, int serverId);
    int getTmClientProfile(int profileId, int serverId);
    int setTmClientProfile(QTmClientProfile *clientProfile, int profileId, int serverId);

    int vncStartClient();
    int vncConnectClient(char* appId, int clientId);
    int vncDisconnectClient(int clientId);
    int vncStopClient(int clientId);
    int vncSetColorFormat(int colorFormat, int clientId);
    int vncSetRunLengthEncoding(bool enable, int clientId);
    int vncSetIncrementalUpdate(bool enable, int clientId);
    int vncSetPreferredSize(int width, int height, bool scaling, int clientid, double clientSinglePixelWidth, double clientSinglePixelHeight, bool considerPixelComp);
    int vncOnFBReadComplete(int bufferId);
    int vncClickVirtualKey(int clientId, int keySym);
    int vncPointerEvent(int clientId, int x, int y, unsigned int pressed);
    int vncSupportedKeyEvents(int clientId, int knobKey, int deviceKey, int mediaKey);
    int vncKeyEvent(int clientId, int keyId, bool pressed);
    int vncBlockFramebuffer(int x, int y, int width, int height, int appId, int reason, int id);
    int vncResetFbUpdateRequestTimer();
    int vncPauseFbUpdateRequest();
    int vncResumeFbUpdateRequest();
    /** ------------------------------------------------------------------------------------------- **/


    /** ------------   slots    --------------------**/
    void vncResized(QSize size, int clientId);
    void vncClientContextInformation(QRect area, int appId, QTmAppInfo *appCategory, QTmDisplayInfo *conCategory, int clientId);
    void vncDisconnected(int clientId);
    void vncConnected(int clientId);

    /** THESE SLOTS HAVE NOT BEEN CONNECTED */
    void vncClientDisplayConfiguration(int clientId);
    void vncClientEventConfiguration(int clientId, int knobKey, int deviceKey, int mediaKey);
    void vncClientKeyboardTrigger(QPoint point, bool remove, int clientId);
    void vncClientKeyEventList(QList<unsigned int> *list, unsigned int counter, int clientId);
    void vncClientFbAlternativeText(unsigned int appId, QString text, int clientId);
    /** END */

    void upnpApplicationAdded(QTmApplication*);
    void upnpApplicationStarted(QTmApplication *application);
    void upnpApplicationTerminated(QTmApplication *application);
    void upnpApplicationStatusChanged(QTmApplication *application);
    void upnpApplicationListingChanged(QTmRemoteServer *remoteServer);
    void remoteServerDeviceAdded(QTmRemoteServer *remoteServer);
    void remoteServerDeviceRemoved(QTmRemoteServer *remoteServer);
};

#endif // LIBINTERFACEIMPL_H
