/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/

#ifndef QTMSURFACEIMPL_H
#define QTMSURFACEIMPL_H
#include <TmHeaders/QTmSurface.h>
#include <TmHeaders/QTmAppInfo.h>
#include <TmHeaders/QTmDisplayInfo.h>
#include <SurfaceObserver.h>

#include <FBManagerInterface.h>
#include <FrameBuffer.h>
#include <FBManagerStates.h>
#include <QVector>
#include <QMutex>

class FrameBuffer;

class AppInterface;

class QTmSurfaceImpl:public QTmSurface, FBManager
{

private:
    int             m_clientId;
    AppInterface*   m_appInterface;
    uchar*          m_framebuffer;
    int             m_bufferSize;
    bool            m_allocated;

    QMutex m_mutex;

    void deleteFrameBuffer();
    static QTmSurfaceImpl* m_instance;
    QTmSurfaceImpl();
    // states of the state machine
    IdleBMState *mIdleBMState;
    AwaitingFBUpdateBMState *mAwaitingFBUpdateBMState;

    FrameBufferManagerState *mCurrMLBMgrState;
    int mBuffCount;
    QVector<FrameBuffer*> mBuffersArray;
    QVector<SurfaceObserver *> mObserverArray;
    void notify();

    FrameBuffer* getFrameBufferById(int bufferId);
    double getCurrMillisec();
public:
    static QTmSurfaceImpl*    getInstance();
    ~QTmSurfaceImpl();

    int getClientId();
    void setClientId(int clientId);
    uchar* allocateSurface(const QSize& s, QTmGlobal::TmColorFormat colorFormat, double xScalingFactor, double yScalingFactor);
    void deallocateSurface();
    void updateSurface(const QRect& updateRect);
    void setAppInterface(AppInterface* appInterface);

    void start();

    void onBufferReadComplete(int bufferId);

    FrameBuffer * getFrameBufferByState(FB_STATE buffState);

    /*************************** state machine methods ***************************/
    FrameBufferManagerState* getIdleBMState();

    FrameBufferManagerState* getAwaitingFBUpdateBMState();

    void setCurrentState(FrameBufferManagerState* state);

    void onBufferNormalStateChange();

    void onBufferAbnormalStateChange(FrameBuffer buffer);

    void startWritingToIdleBuffer();

    uchar* getSurface();
    /*************************** state machine methods ***************************/
    void registerObserver(SurfaceObserver *);
    void resetSurface();
};

#endif // QTMSURFACEIMPL_H
