/*******************************************************************************
**
** Copyright 2013 Sony Corporation
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License, version 2.1, as published by the Free Software Foundation.
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**
*******************************************************************************/
/*
 * ChangeLog -  
 *				March 19, 2013
 * 				1. Added support to enable/disable the consideration of pixel compensation of client display panel.
 *              2. Added suooprt for integrating the double buffer implementation.
 */
#include <libinterfaceemitter.h>
#include <QTmGlobal.h>
#include <TmHeaders/appinterface.h>
#include <qdebug.h>
#include <QTmClientProfile.h>



LibInterfaceEmitter::LibInterfaceEmitter(QObject *parent) :
    QObject(parent)
{
    // Get the Current Version of TM Library.
    st_tm_lib_vrsn verInfo = getTmLibVerInfo();
}

int LibInterfaceEmitter::upnpStartControlPoint()
{
    qDebug()<<"About to emit upnpStartControlPoint()";
    emit sig_upnpStartControlPoint();

    qDebug()<<"sig_upnpStartControlPoint emmited ...";
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::upnpConnectControlPoint(char* interfaceName, char* address)
{
    qDebug()<<"About to emit upnpConnectControlPoint()";
    emit sig_upnpConnectControlPoint(interfaceName, address);
    qDebug()<<"sig_upnpConnectControlPoint emmited ...";
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::upnpStopControlPoint()
{
    qDebug()<<"About to emit upnpStopControlPoint()";
    emit sig_upnpStopControlPoint();
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::upnpDisconnectControlPoint(char* hostAddress)
{
    qDebug()<<"About to emit upnpDisconnectControlPoint()";
    emit sig_upnpDisconnectControlPoint(hostAddress);
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::upnpGetApplicationStatus(char*  appId)
{
    qDebug()<<"LibInterfaceEmitter::upnpGetApplicationStatus()"<< appId;
    emit sig_upnpLaunchApplication(appId);
    return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::upnpLaunchApplication(char* appId)
{
    qDebug()<<"LibInterfaceEmitter::upnpLaunchApplication()"<< appId;
    emit sig_upnpLaunchApplication(appId);
    return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::upnpTerminateApplication(char* appId)
{
    qDebug()<<"LibInterfaceEmitter::upnpTerminateApplication"<< appId;
    emit sig_upnpTerminateApplication(appId);
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::verifyTmClientProfile(QTmClientProfile *clientProfile, int profileId, int serverId)
{
    qDebug() << "About to emit verifyTmClientProfile()";
    qDebug()<<"1. client name :" << clientProfile->clientName() ;
    qDebug()<<"2. client id :" << clientProfile->clientId() ;
    qDebug()<<"3. manufacturer  :" << clientProfile->manufacturerName() ;
    qDebug()<<"4. model no :" << clientProfile->modelNumber();

    qDebug()<<"rtp ipl :" << clientProfile->rtpipl();
    qDebug()<<"rtp mpl :" << clientProfile->rtpmpl();
    qDebug()<<"rtp payload :" << clientProfile->rtppayload();

    emit sig_verifyTmClientProfile(clientProfile,profileId,serverId);
    return AppInterface::SUCCESS;

}
int LibInterfaceEmitter::getTmClientProfile(int profileId, int serverId)
{
    qDebug() << "About to emit getTmClientProfile()";
    emit sig_getTmClientProfile(profileId,serverId);
    return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::setTmClientProfile(QTmClientProfile *clientProfile, int profileId, int serverId)
{
    qDebug() << "About to emit setTmClientProfile()";
    emit sig_setTmClientProfile(clientProfile,profileId,serverId);
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::vncStartClient()
{
    qDebug() << "About to emit vncStartClient()";
    emit sig_vncStartClient();
 return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::vncConnectClient(char* appId, int clientId)
{
    qDebug() << "About to emit setTmClientProfile()";
    emit sig_vncConnectClient(appId,clientId);
    return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::vncDisconnectClient(int clientId)
{
    qDebug() << "About to emit vncDisconnectClient()";
    emit sig_vncDisconnectClient(clientId);
    return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::vncStopClient(int clientId)
{
    qDebug() << "About to emit vncStopClient()";
    emit sig_vncStopClient(clientId);
    return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::vncSetColorFormat(int colorFormat, int clientId)
{
    qDebug() << "About to emit vncSetColorFormat()";
    emit sig_vncSetColorFormat(colorFormat,clientId);
    return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::vncSetRunLengthEncoding(bool enable, int clientId)
{
    qDebug() << "About to emit vncSetRunLengthEncoding()";
    emit sig_vncSetRunLengthEncoding(enable,clientId);
    return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::vncSetIncrementalUpdate(bool enable, int clientId)
{
    qDebug() << "About to emit vncSetIncrementalUpdate()";
    emit sig_vncSetIncrementalUpdate(enable,clientId);
    return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::vncSetPreferredSize(int width, int height, bool scaling, int clientid, double clientSinglePixelWidth, double clientSinglePixelHeight, bool considerPixelComp)
{
    qDebug() << "About to emit vncSetPreferredSize()";
    emit sig_vncSetPreferredSize(width,height,scaling,clientid, clientSinglePixelWidth, clientSinglePixelHeight, considerPixelComp);
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::vncOnFBReadComplete(int bufferId)
{
    qDebug() << "About to emit vncOnFBReadComplete()";
    emit sig_vncOnFBReadComplete(bufferId);
}

int LibInterfaceEmitter::vncClickVirtualKey(int clientId, int keySym)
{
    qDebug() << "About to emit vncClickVirtualKey()";
    emit sig_vncClickVirtualKey(clientId,keySym);
    return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::vncPointerEvent(int clientId, int x, int y, unsigned int pressed)
{
    qDebug() << "About to emit vncPointerEvent()";
    emit sig_vncPointerEvent(clientId,x,y,pressed);
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::vncSupportedKeyEvents(int clientId, int knobKey, int deviceKey, int mediaKey)
{
    qDebug() << "About to emit vncKeyEvent()";
    emit sig_vncSupportedKeyEvents(clientId, knobKey, deviceKey, mediaKey);
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::vncKeyEvent(int clientId, int keyId, bool pressed)
{
    qDebug() << "About to emit vncKeyEvent()";
    emit sig_vncKeyEvent(clientId,keyId,pressed);
    return AppInterface::SUCCESS;
}
int LibInterfaceEmitter::vncBlockFramebuffer(int x, int y, int width, int height, int appId, int reason, int id)
{
    qDebug() << "About to emit vncBlockFramebuffer()";
    emit sig_vncBlockFramebuffer(x,y,width,height,appId,reason,id);
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::vncResetFbUpdateRequestTimer()
{
    qDebug() << "About to emit vncResetFbUpdateRequestTimer()";
    emit sig_vncResetFbUpdateRequestTimer();
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::vncPauseFbUpdateRequest()
{
    qDebug() << "About to emit vncPauseFbUpdateRequest()";
    emit sig_vncPauseFbUpdateRequest();
    return AppInterface::SUCCESS;
}

int LibInterfaceEmitter::vncResumeFbUpdateRequest()
{
    qDebug() << "About to emit vncResumeFbUpdateRequest()";
    emit sig_vncResumeFbUpdateRequest();
    return AppInterface::SUCCESS;
}

st_tm_lib_vrsn LibInterfaceEmitter::getTmLibVerInfo()
{
#ifdef QT_CODE_DEBUG
    qDebug()<< "****************** Terminal Mode QT Library Version Info ************************";
#endif //QT_CODE_DEBUG
 /* Fill the Structure members with LIBRARY version info */
    st_tm_lib_vrsn libVerPtr;
    libVerPtr.major    = TM_LIBVER_MAJOR;
    libVerPtr.minor    = TM_LIBVER_MINOR;
    libVerPtr.revision = TM_LIBVER_REVISION;
    libVerPtr.purpose  = TM_LIBVER_PURPOSE;
    /** Print The current Library Version Info **/

#ifdef QT_CODE_DEBUG
    qDebug() << "Terminal Mode Library :: Major Version :" << libVerPtr.major;
    qDebug() << "Terminal Mode Library :: Minor Version :" << libVerPtr.minor;
    qDebug() << "Terminal Mode Library :: Revision Version :" << libVerPtr.revision;
    qDebug() << "Terminal Mode Library :: Purpose Version :" << libVerPtr.purpose;
    qDebug() << "Terminal Mode Library :: Version : " << libVerPtr.major << "."<< libVerPtr.minor<<"."<<libVerPtr.revision<<"."<<libVerPtr.purpose;
#endif //QT_CODE_DEBUG

    return libVerPtr;

}
